searchData={"items":[{"type":"module","title":"Bundlex","doc":"Some common utility functions.","ref":"Bundlex.html"},{"type":"function","title":"Bundlex.build_path/3","doc":"Returns path where compiled native is stored.","ref":"Bundlex.html#build_path/3"},{"type":"function","title":"Bundlex.family/0","doc":"Returns family of the platform obtained with `platform/0`.","ref":"Bundlex.html#family/0"},{"type":"function","title":"Bundlex.get_target/0","doc":"A function returning information about the target platform. In case of cross-compilation the\ninformation can be provided by setting appropriate environment variables.","ref":"Bundlex.html#get_target/0"},{"type":"function","title":"Bundlex.platform/0","doc":"Returns current platform name.","ref":"Bundlex.html#platform/0"},{"type":"type","title":"Bundlex.platform_t/0","doc":"","ref":"Bundlex.html#t:platform_t/0"},{"type":"type","title":"Bundlex.target/0","doc":"A map containing four fields that describe the target platform.\n\nIt consists of:\n* architecture - e.g. `x86_64` or `arm64`\n* vendor - e.g. `pc`\n* os - operating system, e.g. `linux` or `darwin20.6.0`\n* abi - application binary interface, e.g. `musl` or `gnu`","ref":"Bundlex.html#t:target/0"},{"type":"module","title":"Bundlex.CNode","doc":"Utilities to ease interaction with Bundlex-based CNodes, so they can be treated\nmore like Elixir processes / `GenServer`s.","ref":"Bundlex.CNode.html"},{"type":"function","title":"Bundlex.CNode.call/3","doc":"Makes a synchronous call to CNode and waits for its reply.\n\nThe CNode is supposed to send back a `{cnode, response}` tuple where `cnode`\nis the node name of CNode. If the response doesn't come in within `timeout`,\nerror is raised.\n\nMessages are exchanged directly (without interacting with CNode's associated\nserver).","ref":"Bundlex.CNode.html#call/3"},{"type":"function","title":"Bundlex.CNode.monitor/1","doc":"Starts monitoring CNode from the calling process.","ref":"Bundlex.CNode.html#monitor/1"},{"type":"function","title":"Bundlex.CNode.send/2","doc":"Sends a message to cnode.\n\nThe message is exchanged directly (without interacting with CNode's associated\nserver).","ref":"Bundlex.CNode.html#send/2"},{"type":"macro","title":"Bundlex.CNode.start/1","doc":"Spawns and connects to CNode `cnode_name` from application of calling module.\n\nSee `Bundlex.CNode.start/2` for more details.","ref":"Bundlex.CNode.html#start/1"},{"type":"function","title":"Bundlex.CNode.start/2","doc":"Works the same way as `start_link/2`, but does not link to CNode's associated\nserver.","ref":"Bundlex.CNode.html#start/2"},{"type":"macro","title":"Bundlex.CNode.start_link/1","doc":"Spawns and connects to CNode `cnode_name` from application of calling module.\n\nSee `Bundlex.CNode.start_link/2` for more details.","ref":"Bundlex.CNode.html#start_link/1"},{"type":"function","title":"Bundlex.CNode.start_link/2","doc":"Spawns and connects to CNode `cnode_name` from application `app`.\n\nThe CNode is passed the following command line arguments:\n- host name,\n- alive name,\n- node name,\n- creation number.\n\nAfter CNode startup, these parameters should be passed to\n[`ei_connect_xinit`](http://erlang.org/doc/man/ei_connect.html#ei_connect_xinit)\nfunction, and CNode should be published and await connection. Once the CNode is\npublished, it should print a line starting with `ready` to the standard output\n**and flush the standard output** to avoid the line being buffered.\n\nUnder the hood, this function starts an associated server, which is responsible\nfor monitoring the CNode and monitoring calling process to be able to do proper\ncleanup upon a crash. On startup, the server does the following:\n1. Makes current node distributed if it is not done yet (see `Node.start/3`).\n1. Assigns CNode a unique name.\n1. Starts CNode OS process using `Port.open/2`.\n1. Waits (at most 5 seconds) until a line `ready` is printed out\n(this line is captured and not forwarded to the stdout).\n1. Connects to the CNode.\n\nThe erlang cookie is passed using the BUNDLEX_ERLANG_COOKIE an environment variable.","ref":"Bundlex.CNode.html#start_link/2"},{"type":"function","title":"Bundlex.CNode.stop/1","doc":"Disconnects from CNode.\n\nIt is the responsibility of the CNode to exit upon connection loss.","ref":"Bundlex.CNode.html#stop/1"},{"type":"type","title":"Bundlex.CNode.on_start_t/0","doc":"","ref":"Bundlex.CNode.html#t:on_start_t/0"},{"type":"type","title":"Bundlex.CNode.t/0","doc":"Reference to the CNode.\n\nConsists of pid of CNode's associated server and CNode name.","ref":"Bundlex.CNode.html#t:t/0"},{"type":"module","title":"Bundlex.Doxygen","doc":"Module responsible for generating doxygen documentation for Bundlex projects.","ref":"Bundlex.Doxygen.html"},{"type":"function","title":"Bundlex.Doxygen.doxygen/1","doc":"Prepares struct with all necessary filepaths for the native documentation","ref":"Bundlex.Doxygen.html#doxygen/1"},{"type":"function","title":"Bundlex.Doxygen.generate_doxyfile/1","doc":"Generates doxyfile in the c_src/project directory for Bundlex project.","ref":"Bundlex.Doxygen.html#generate_doxyfile/1"},{"type":"function","title":"Bundlex.Doxygen.generate_doxygen_documentation/1","doc":"Generates html doxygen documentation for the Bundlex project. Doxyfile must be generated before.","ref":"Bundlex.Doxygen.html#generate_doxygen_documentation/1"},{"type":"function","title":"Bundlex.Doxygen.generate_hex_page/1","doc":"Generates page for the Bundlex project in the pages/doxygen directory.\nPage must be manually added to the docs extras in the mix.exs.\nPage contains only link to the doxygen html documentation.","ref":"Bundlex.Doxygen.html#generate_hex_page/1"},{"type":"type","title":"Bundlex.Doxygen.doxygen_t/0","doc":"","ref":"Bundlex.Doxygen.html#t:doxygen_t/0"},{"type":"module","title":"Bundlex.Loader","doc":"Some utilities to ease loading of Bundlex-based NIFs.","ref":"Bundlex.Loader.html"},{"type":"macro","title":"Bundlex.Loader.__using__/1","doc":"Binds the module to the specified NIF.\n\nAccepts keyword list, that may contain two arguments:\n- `:nif` - required, name of the NIF to be bound (the same as specified in `bundlex.exs`)\n- `:app` - application defining the NIF, defaults to the current application\n\nAfter `use`'ing this module you can utilize `defnif/1` and `defnifp/1` macros\nto create bindings to particular native functions.","ref":"Bundlex.Loader.html#__using__/1"},{"type":"macro","title":"Example - Bundlex.Loader.__using__/1","doc":"defmodule My.Native.Module do\n      use Bundlex.Loader, nif: :my_nif\n\n      defnif native_function(arg1, arg2, arg3)\n\n      def normal_function(arg1, arg2) do\n        # ...\n      end\n\n      defnifp private_native_function(arg1, arg2)\n\n    end","ref":"Bundlex.Loader.html#__using__/1-example"},{"type":"macro","title":"Bundlex.Loader.defnif/1","doc":"Generates function bound to the native implementation. This module has to be\n`use`d for this macro to work.\n\nFunction name should correspond to the native one.\n\nSee `__using__/1` for examples.","ref":"Bundlex.Loader.html#defnif/1"},{"type":"macro","title":"Bundlex.Loader.defnifp/1","doc":"Works the same way as `defnif/1`, but generates private function. This module\nhas to be `use`d for this macro to work.\n\nSee `__using__/1` for examples.","ref":"Bundlex.Loader.html#defnifp/1"},{"type":"macro","title":"Bundlex.Loader.load_nif!/2","doc":"Binds calling module to NIF `nif_name` from application `app`.\n\nSecond argument has to be an atom, the same as name of the NIF in the bundlex\nproject.\n\nInvoked internally by `__using__/1` macro, which is the preferred way of loading\nNIFs.","ref":"Bundlex.Loader.html#load_nif!/2"},{"type":"module","title":"Bundlex.Native","doc":"Module responsible for parsing and processing natives' configurations.","ref":"Bundlex.Native.html"},{"type":"function","title":"Bundlex.Native.resolve_natives/2","doc":"Parses natives and generates compiler commands.","ref":"Bundlex.Native.html#resolve_natives/2"},{"type":"type","title":"Bundlex.Native.t/0","doc":"","ref":"Bundlex.Native.html#t:t/0"},{"type":"module","title":"Bundlex.Port","doc":"Utilities to ease interaction with Ports.","ref":"Bundlex.Port.html"},{"type":"macro","title":"Bundlex.Port.open/2","doc":"Spawns Port `native_name` from application of calling module.\nReturned result is compatible with standard Port API.","ref":"Bundlex.Port.html#open/2"},{"type":"function","title":"Bundlex.Port.open/3","doc":"","ref":"Bundlex.Port.html#open/3"},{"type":"behaviour","title":"Bundlex.Project","doc":"Behaviour that should be implemented by each project using Bundlex in the\n`bundlex.exs` file.","ref":"Bundlex.Project.html"},{"type":"function","title":"Bundlex.Project.get/1","doc":"Returns the project struct of given application.\n\nIf the module has not been loaded yet, it is loaded from\n`project_dir/bundlex.exs` file.","ref":"Bundlex.Project.html#get/1"},{"type":"function","title":"Bundlex.Project.native_config_keys/0","doc":"","ref":"Bundlex.Project.html#native_config_keys/0"},{"type":"callback","title":"Bundlex.Project.project/0","doc":"Callback returning project configuration.","ref":"Bundlex.Project.html#c:project/0"},{"type":"function","title":"Bundlex.Project.project_module?/1","doc":"Determines if `module` is a bundlex project module.","ref":"Bundlex.Project.html#project_module?/1"},{"type":"type","title":"Bundlex.Project.config/0","doc":"Type describing input project configuration.\n\nIt's a keyword list, where natives and libs can be specified. Libs are\nnative packages that are compiled as static libraries and linked to natives\nthat have them specified in `deps` field of their configuration.","ref":"Bundlex.Project.html#t:config/0"},{"type":"type","title":"Bundlex.Project.native_config/0","doc":"Type describing configuration of a native.\n\nConfiguration of each native may contain following options:\n* `sources` - C files to be compiled (at least one must be provided).\n* `preprocessors` - Modules that will pre-process the native. They may change this configuration, for example\nby adding new keys. An example of preprocessor is [Unifex](https://hexdocs.pm/unifex/Unifex.html).\nSee `Bundlex.Project.Preprocessor` for more details.\n* `interface` - Interface used to integrate with Elixir code. The following interfaces are available:\n  * :nif - dynamically linked to the Erlang VM (see [Erlang docs](http://erlang.org/doc/man/erl_nif.html))\n  * :cnode - executed as separate OS processes, accessed through sockets (see [Erlang docs](http://erlang.org/doc/man/ei_connect.html))\n  * :port - executed as separate OS processes (see [Elixir Port docs](https://hexdocs.pm/elixir/Port.html))\nSpecifying no interface is valid only for libs.\n* `deps` - Dependencies in the form of `{app, lib_name}`, where `app`\nis the application name of the dependency, and `lib_name` is the name of lib\nspecified in Bundlex project of this dependency. Empty list by default. See _Dependencies_ section below\nfor details.\n* `os_deps` - List of external OS dependencies. It's a keyword list, where each key is the\ndependency name and the value is a provider or a list of them. In the latter case, subsequent\nproviders from the list will be tried until one of them succeeds. A provider may be one of:\n  - `pkg_config` - Resolves the dependency via `pkg-config`. Can be either `{:pkg_config, pkg_configs}`\n  or just `:pkg_config`, in which case the dependency name will be used as the pkg_config name.\n  - `precompiled` - Downloads the dependency from a given url and sets appropriate compilation\n  and linking flags. Can be either `{:precompiled, url, libs}` or `{:precompiled, url}`, in which\n  case the dependency name will be used as the lib name.\n  Precompiled dependencies can be disabled via configuration globally:\n\n    ```elixir\n    config :bundlex, :disable_precompiled_os_deps, true\n    ```\n\n    or for given applications (Mix projects), for example:\n\n    ```elixir\n    config :bundlex, :disable_precompiled_os_deps,\n      apps: [:my_application, :another_application]\n    ```\n\n    Note that this will affect the natives and libs defined in the `bundlex.exs` files of specified\n    applications only, not in their dependencies.\n\n  Check `t:os_dep/0` for details.\n* `pkg_configs` - (deprecated, use `os_deps` instead) Names of libraries for which the appropriate flags will be\nobtained using pkg-config (empty list by default).\n* `language` - Language of native. `:c` or `:cpp` may be chosen (`:c` by default).\n* `src_base` - Native files should reside in `project_root/c_src/ `\n(application name by default).\n* `includes` - Paths to look for header files (empty list by default).\n* `lib_dirs` - Absolute paths to look for libraries (empty list by default).\n* `libs` - Names of libraries to link (empty list by default).\n* `compiler_flags` - Custom flags for compiler. Default `-std` flag for `:c` is `-std=c11` and for `:cpp` is `-std=c++17`.\n* `linker_flags` - Custom flags for linker.","ref":"Bundlex.Project.html#t:native_config/0"},{"type":"type","title":"Bundlex.Project.native_interface/0","doc":"","ref":"Bundlex.Project.html#t:native_interface/0"},{"type":"type","title":"Bundlex.Project.native_language/0","doc":"","ref":"Bundlex.Project.html#t:native_language/0"},{"type":"type","title":"Bundlex.Project.native_name/0","doc":"","ref":"Bundlex.Project.html#t:native_name/0"},{"type":"type","title":"Bundlex.Project.os_dep/0","doc":"","ref":"Bundlex.Project.html#t:os_dep/0"},{"type":"type","title":"Bundlex.Project.os_dep_provider/0","doc":"","ref":"Bundlex.Project.html#t:os_dep_provider/0"},{"type":"type","title":"Bundlex.Project.t/0","doc":"Struct representing bundlex project.\n\nContains the following fields:\n- `:config` - project configuration\n- `:src_path` - path to the native sources\n- `:module` - bundlex project module\n- `:app` - application that exports project","ref":"Bundlex.Project.html#t:t/0"},{"type":"behaviour","title":"Bundlex.Project.Preprocessor","doc":"Behaviour for preprocessing Bundlex projects.\n\nPrecompiling may involve either generating additional resources or altering the project itself.\nCurrently, preprocessing native configuration (`c:preprocess_native_config/3`)\nand parsed natives (`c:preprocess_native/1`) is supported.","ref":"Bundlex.Project.Preprocessor.html"},{"type":"callback","title":"Bundlex.Project.Preprocessor.preprocess_native/1","doc":"","ref":"Bundlex.Project.Preprocessor.html#c:preprocess_native/1"},{"type":"callback","title":"Bundlex.Project.Preprocessor.preprocess_native_config/3","doc":"","ref":"Bundlex.Project.Preprocessor.html#c:preprocess_native_config/3"},{"type":"type","title":"Bundlex.Project.Preprocessor.t/0","doc":"","ref":"Bundlex.Project.Preprocessor.html#t:t/0"},{"type":"task","title":"mix bundlex.doxygen","doc":"Generates doxygen documentation for Bundlex project\n\nAccepts the following command line arguments:\n- `--yes`, `-y` - skips confirmation prompt and overwrites existing meta files\n- `--no`, `-n` - skips confirmation prompt and does not overwrite existing meta files","ref":"Mix.Tasks.Bundlex.Doxygen.html"},{"type":"task","title":"mix compile.bundlex","doc":"Builds natives specified in bundlex.exs file\n\nAccepts the following command line arguments:\n- `--store-scripts` - if set, shell scripts are stored in the project\nroot folder for further analysis.\n\nAdd `:bundlex` to compilers in your Mix project to have this task executed\neach time the project is compiled.","ref":"Mix.Tasks.Compile.Bundlex.html"},{"type":"exception","title":"Bundlex.Doxygen.Error","doc":"","ref":"Bundlex.Doxygen.Error.html"},{"type":"exception","title":"BundlexError","doc":"","ref":"BundlexError.html"},{"type":"extras","title":"Bundlex","doc":"# Bundlex\n\n[![Hex.pm](https://img.shields.io/hexpm/v/bundlex.svg)](https://hex.pm/packages/bundlex)\n[![API Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/bundlex/)\n[![CircleCI](https://circleci.com/gh/membraneframework/bundlex.svg?style=svg)](https://circleci.com/gh/membraneframework/bundlex)\n\nBundlex is a multi-platform tool for compiling C and C++ code along with elixir projects, for use in NIFs, CNodes and Ports. The tool also provides a convenient way of accessing compiled code in elixir modules.\n\nBundlex has been tested on Linux, Mac OS and FreeBSD. There's some support for Windows as well, but it's experimental and unstable (see issues for details).\n\nBundlex also supports cross-compilation and has been tested with platforms running Nerves.\n\nThis tool is maintained by the [Membrane Framework](https://membraneframework.org/) team.","ref":"readme.html"},{"type":"extras","title":"Installation - Bundlex","doc":"To install, you need to configure Mix project as follows:\n\n```elixir\ndefmodule MyApp.Mixfile do\n  use Mix.Project\n\n  def project do\n    [\n      app: :my_app,\n      compilers: [:bundlex] ++ Mix.compilers, # add bundlex to compilers\n      deps: deps(),\n      # ...\n   ]\n  end\n\n  defp deps() do\n    [\n      {:bundlex, \"~> 1.5\"}\n    ]\n  end\nend\n```\n\nand create `bundlex.exs` file in the project root folder, containing Bundlex project module:\n\n```elixir\ndefmodule MyApp.BundlexProject do\n  use Bundlex.Project\n\n  def project() do\n    []\n  end\nend\n```\n\nNow your project does not contain any C sources, but should compile successfully, and some Bundlex messages should be printed while compilation proceeds.","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Bundlex","doc":"#","ref":"readme.html#usage"},{"type":"extras","title":"Adding natives to project - Bundlex","doc":"Adding natives can be done in `project/0` function of Bundlex project module in the following way:\n\n```elixir\ndefmodule MyApp.BundlexProject do\n  use Bundlex.Project\n\n  def project() do\n    [\n      natives: natives(Bundlex.platform),\n      libs: libs()\n    ]\n  end\n\n  defp natives(:linux) do\n    [\n      my_native: [\n        sources: [\"something.c\", \"linux_specific.c\"],\n        interface: :nif\n      ],\n      my_other_native: [\n        sources: [\"something_other.c\", \"linux_specific.c\"],\n        interface: :cnode\n      ],\n      my_other_native: [\n        sources: [\"something_more_other.c\", \"linux_specific.c\"],\n        interface: :port\n      ]\n    ]\n  end\n\n  defp natives(_platform) do\n    [\n      my_native: [\n        sources: [\"something.c\", \"multiplatform.c\"],\n        interface: :nif\n      ],\n      my_other_native: [\n        sources: [\"something_other.c\", \"multiplatform.c\"],\n        interface: :cnode\n      ],\n      my_other_native: [\n        sources: [\"something_more_other.c\", \"multiplatform.c\"],\n        interface: :port\n      ]\n    ]\n  end\n\n  defp libs() do\n    [\n      my_lib: [\n        sources: [\"something.c\"],\n        interface: :nif\n      ],\n      my_lib: [\n        sources: [\"something_other.c\"],\n        interface: :cnode\n      ]\n    ]\n  end\nend\n```\n\nAs we can see, we can specify two types of resources:\n- natives - code implemented in C that will be used within Elixir code\n- libs - can be used by natives or other libs as [dependencies](#Dependencies)\n\nBy default, the sources should reside in `project_root/c_src/my_app` directory.\n\nFor more details and available options, see [Bundlex.Project.native_config](https://hexdocs.pm/bundlex/Bundlex.Project.html#t:native_config/0).\n\n#","ref":"readme.html#adding-natives-to-project"},{"type":"extras","title":"Dependencies - Bundlex","doc":"Each native can have dependencies - libs that are statically linked to it and can be included in its native code like `#include lib_name/some_header.h`. The following rules apply:\n- To add dependencies from a separate project, it must be available via Mix. \n- Only libs can be added as dependencies.\n- Each dependency of a native must specify the same or no interface. If there exist multiple versions of dependency with different interfaces, the proper version is selected automatically.\n- A lib that specifies no interface can depend on libs with no interfaces only.\n\n#","ref":"readme.html#dependencies"},{"type":"extras","title":"Compilation options - Bundlex","doc":"The following command-line arguments can be passed:\n- `--store-scripts` - if set, shell scripts are stored in the project\nroot folder for further analysis.\n\n#","ref":"readme.html#compilation-options"},{"type":"extras","title":"Loading NIFs in modules - Bundlex","doc":"NIFs compiled with Bundlex can be loaded the same way as any other NIFs (see [`:erlang.load_nif/2`](http://erlang.org/doc/man/erlang.html#load_nif-2)), but Bundlex provides `Bundlex.Loader` module to save you some boilerplate:\n\n```elixir\ndefmodule MyApp.SomeNativeStuff do\n  use Bundlex.Loader, nif: :my_nif\n\n  def normal_function(a, b, c, d) do\n    private_native_function(a+b, c+d)\n  end\n\n  defnif native_function(a, b)\n\n  defnifp private_native_function(x, y)\n\nend\n```\n\nNote that unlike when using `:erlang.load_nif/2`, here `def`s and `defp`s can be used to create usual functions, native ones are declared with `defnif` and `defnifp`. This is achieved by creating a new module under the hood, and that is why the module passed to C macro `ERL_NIF_INIT` has to be succeeded by `.Nif`, i.e.\n```C\nERL_NIF_INIT(MyApp.SomeNativeStuff.Nif, funs, load, NULL, upgrade, unload)\n```\n\nDespite this, any native erlang macros and functions shall be used as usual, as described at http://erlang.org/doc/man/erl_nif.html\n\n#","ref":"readme.html#loading-nifs-in-modules"},{"type":"extras","title":"Interacting with CNodes - Bundlex","doc":"As in the case of NIFs, CNodes compiled with Bundlex can be used like any other CNodes (see built-in `Node` module), while some useful stuff for interacting with them is provided. `Bundlex.CNode` module contains utilities that make it easier to spawn and control CNodes, and allow them to treat them more like usual Elixir processes. Check out the documentation for more details.\n\n#","ref":"readme.html#interacting-with-cnodes"},{"type":"extras","title":"Interacting with Ports - Bundlex","doc":"Similarly to CNodes Bundlex provides `Bundlex.Port` module for a little easier interacting with Ports.\nPlease refer to the module's documentation to see how to use it.\n\n#","ref":"readme.html#interacting-with-ports"},{"type":"extras","title":"Cross-compilation - Bundlex","doc":"With proper setup, Bundlex can support cross-compilation. When using Nerves it should work out of the box. \n\nNot relying on Nerves and using your own toolchain is also possible, although it wasn't tested. In this scenario, the following environment variables need to be set during compilation (when changing the target bundlex also needs to be recompiled):\n- `CROSSCOMPILE` - value is not important, just needs to be set\n- `CC` - path to the C compiler for cross-compiling to the target\n- `CFLAGS` - C compilation flags\n- `CXX` - path to the C++ compiler for cross-compiling to the target \n- `CXXFLAGS` - C++ compilation flags\n- `LDFLAGS` - Linker flags\n\nIf you wish for `Bundlex.get_target/0` to return accurate information about your target, set the following environment variables:\n- `TARGET_ARCH` - The target CPU architecture (e.g., `arm`, `aarch64`, `mipsel`, `x86_64`, `riscv64`)\n- `TARGET_VENDOR` - Vendor of your target platform\n- `TARGET_OS` - The targes OS (e.g. `linux`)\n- `TARGET_ABI` - The target ABI (e.g., `gnueabihf`, `musl`)\n\nWhen cross-compiling some warnings may be raised about not being able to load nifs, but that's expected, since they are most likely built for different architecture.\n\n#","ref":"readme.html#cross-compilation"},{"type":"extras","title":"Documentation of the native code - Bundlex","doc":"Bundlex provides a way to generate documentation of the native code. The documentation is generated using [Doxygen](http://www.doxygen.nl/).\n\nTo do so, run `$ mix bundlex.doxygen` command. The documentation is generated for each native separately. The documentation of the native `project_name` will be generated in `doc/bundlex/project_name` directory. Additionally, hex doc page with the link to the Doxygen documentation is generated in the `pages/doxygen/project_name.md` and should be included in the `mix.exs` file: \n\n```elixir\ndefp docs do\n  [\n    extras: [\n      \"pages/doxygen/project_name.md\",\n      ...\n    ],\n    ...\n  ]\nend\n```\nIf you want to keep own changes in the `pages/doxygen/project_name.md` file, you can use `--no` option to skip the generation of this file. Otherwise, if you want the file to be always overwritten, use `--yes` option.\n\nAfter that, the documentation can be generated with `mix docs` command.\n\n#","ref":"readme.html#documentation-of-the-native-code"},{"type":"extras","title":"Include native documentation in the hex docs - Bundlex","doc":"To include the native documentation in the hex docs, you need to generate the documentation with `$ mix bundlex.doxygen` command and include hex page in the `extras`, before running `$ mix hex.publish` command.","ref":"readme.html#include-native-documentation-in-the-hex-docs"},{"type":"extras","title":"More examples - Bundlex","doc":"More advanced examples can be found in our [test_projects](https://github.com/membraneframework/bundlex/tree/master/test_projects)\nor in our [repositories](https://github.com/membraneframework) where we use Bundlex e.g. in [Unifex](https://github.com/membraneframework/unifex).","ref":"readme.html#more-examples"},{"type":"extras","title":"Copyright and License - Bundlex","doc":"Copyright 2018, [Software Mansion](https://swmansion.com/?utm_source=git&utm_medium=readme&utm_campaign=membrane)\n\n[![Software Mansion](https://logo.swmansion.com/logo?color=white&variant=desktop&width=200&tag=membrane-github)](https://swmansion.com/?utm_source=git&utm_medium=readme&utm_campaign=membrane)\n\nLicensed under the [Apache License, Version 2.0](LICENSE)","ref":"readme.html#copyright-and-license"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright 2018 Software Mansion\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}